$title	Read the Social Accouting Matrix and Balance

*	A social accounting matrix is a square matrix which summarizes all economic
*	transactions which have taken place in a given year.  Consistency requires
*	that row sums equal column sums.  This program illustrates how consistency
*	can be imposed using either linear or nonlinear programming.  The estimations
*	differ only in the definition of the deviation.

set     i   SAM rows and column indices   /1*124/;

alias  (i,j);

Table sam(i,j)  Base year social accounts
$include TZSAM2001.inc

*	Chech if sum of rows equal sum of columns

parameter       chksam(i,*)       Consistency check of social accounts;
chksam(i,"before") = sum(j, sam(i,j)-sam(j,i));
chksam(i,"scale") = sum(j, sam(i,j));
chksam(i,"%dev")$sum(j, sam(i,j)) = 100 * sum(j, sam(i,j)-sam(j,i)) / sum(j, sam(i,j));

*	Balance the SAM using least squares

variable        obj             Objective -- least squares deviation;

positive
variable        es(i,j)			Estimate of SAM entries
		pdiff(i,j)		Positive difference
		ndiff(i,j)		Negative difference;

equations       lsqobj          Defines norm of the deviation with a least squares objective,
		absobj		Defines norm of the deviation with an absolute deviation objective,
		infobj		Defines norm of the deviation with a box norm objective,
		entobj		Defines norm of the entropy of the difference

                balance(i)      SAM balance condition
		absdiff(i,j)	Defines the absolute deviation for a given element;

entobj..	obj =e= sum((i,j)$sam(i,j), es(i,j) * (log(es(i,j)/sam(i,j))-1));

lsqobj..        obj =e=   sum((i,j)$sam(i,j), sam(i,j) * sqr(es(i,j)/sam(i,j)-1));

absobj..	obj =e= sum((i,j)$sam(i,j), (pdiff(i,j) + ndiff(i,j))/sam(i,j));

infobj(i,j)$sam(i,j)..	obj =g= pdiff(i,j) + ndiff(i,j);

balance(i)..    sum(j, es(i,j)) =e= sum(j, es(j,i));

absdiff(i,j)$sam(i,j)..		pdiff(i,j) - ndiff(i,j) =e= sam(i,j) - es(i,j);


es.l(i,j) = sam(i,j);
es.fx(i,j)$(sam(i,j) = 0) = 0;

model lsqr /lsqobj, balance/;
model absd /absobj, balance, absdiff/;
model infd /infobj, balance, absdiff/;
model entropy /entobj, balance/;

parameter	compare	Comparison of least squares and absolute difference;

option NLP=pathnlp;
solve entropy using NLP minimizing obj;
compare(i,j,"Entropy") = es.l(i,j);

*	A balanced SAM can be computed using nonlinear programming:

option NLP=pathnlp;
solve lsqr using NLP minimizing obj;

compare(i,j,"L2") = es.l(i,j);

*	Alternatively, the balanced SAM can be estimtated using linear programming
*	and minimizing the L1 norm of adjustments:

option lp=bdmlp;
solve infd using LP minimizing obj;
compare(i,j,"Linf") = es.l(i,j);

*	Yet a third alternative is to balance the SAM using linear programming
*	and minimizing the L-infinity norm of adjustments:

option lp=bdmlp;
solve absd using LP minimizing obj;
compare(i,j,"L1") = es.l(i,j);

option compare:3:2:1;
display compare;

chksam(i,"after") = sum(j, sam(i,j)-sam(j,i));
display chksam;

